<?php

class rentPress_ShortCodes_FloorPlans_Featured extends rentPress_ShortCodes_Base
{
    public function formatPricing($cost)
    {
        $fpRentSummary = $cost === 0 ? '<i>No pricing available</i>' : 'Starting at $'.number_format($cost).'/month';
        $fpRentSummary = $cost === -1 ? 'Contact us for pricing' : $fpRentSummary;
        return $fpRentSummary;
    }

    public function handleShortcode($atts, $content = '') 
    {
        global $wpdb;

        $attributes = shortcode_atts( array(
            'floor_plan_code' => false,
            'link_to_availability' => false,
            'intro_title' => false,
            'intro_content' => false
        ), $atts );

        if ( $attributes['floor_plan_code'] ) {
            $wpQueryArgs = [
                'post_type' => 'floorplans',
                'post_status' => 'publish',
                'floorplan_code' => $attributes['floor_plan_code'],
                'posts_per_page' => 1 // Cause this is for a single featured floor plan
            ];

            $floorPlans = new WP_Query($wpQueryArgs);

            ob_start();

            echo '<div id="rp-featured-floor-plan-intro">';
                if ( $attributes['intro_title'] ) {
                    $title = esc_html($attributes['intro_title']);
                    echo "<h1>{$title}</h1>";
                }
                if ( $attributes['intro_content'] ) {
                    $content = esc_html($attributes['intro_content']);
                    echo "<p>{$content}</p>";
                }
            echo '</div>';

            if ( $floorPlans->have_posts() ) {
                global $rentPress_Service;
                echo '<section id="rp-featured-floor-plan-container" class="rp-short-code fp-list">';
                while ( $floorPlans->have_posts() ) { $floorPlans->the_post();
                    $floorPlanService = $rentPress_Service['floorplans_meta']->setPostID($floorPlans->post->ID); ?>
                        <div class="rp-featured-floorplan">
                            <?php $imgLink = $attributes['link_to_availability'] !== false ? $floorPlanService->availabilityUrl() : get_the_permalink(); ?>
                            
                            <figure>
                                <a href="<?= $imgLink ?>">
                                    <img src="<?= $floorPlanService->image() ?>">
                                </a>
                            </figure>

                            <div class="rp-featured-floorplan-details">
                                <h3>
                                    <span><?php echo $floorPlans->post->fpName; ?></span>
                                </h3>
                                <h5>
                                    <span><?= number_format($floorPlans->post->fpMinSQFT) ?> Sq. Ft.</span><br>
                                    <span>Starting at $<?= number_format($floorPlans->post->fpMinRent) ?></span>
                                </h5>
                                <aside>
                                    <span><?= intval($floorPlans->post->fpBeds) == 0 ? 'Studio' : $floorPlans->post->fpBeds . ' Bed' ?></span> | 
                                    <span><?php echo number_format($floorPlans->post->fpBaths) ?>&nbsp;Bath</span>
                                    <div class="units-avail"><?= $floorPlans->post->fpAvailUnitCount ?> Apartments Available</div>
                                    <?php if ( $attributes['link_to_availability'] !== false ) : ?>
                                        <a target="_blank" class="button btn" href="<?php echo $floorPlanService->availabilityUrl(); ?>">View Availability</a>
                                    <?php else : ?>
                                        <a class="button btn" href="<?php the_permalink(); ?>">View Available Apartments</a>
                                    <?php endif; ?>
                                </aside>
                            </div>
                        </div> <!-- .featured-floorplan -->
                    <?php
                }
                echo '</section>';
            } else {
                echo 'No floor plans match the provided criteria. Please check your shortcode parameters.';
            }
            wp_reset_postdata();
        } else {
            echo '<div class="no-featured-fp-found">Sorry! We could not fetch the floor plan because the floor_plan_code short code parameter was not provided</div>';
        }

        return ob_get_clean();
    }
}
